function [nb, deg] = makeClosedRings(tri, n)
% [nb, deg] = makeClosedRings(tri, n)
% for a triangulation with n vertices and all triangles oriented ccw,
% column i of nb has a vertex, followed by its neigbhors in ccw order.
% deg(i) = 0 if ring i is not closed, else
% deg(i) = the # of vertices (one plus # of incident triangles)
%
% The triangles must be canonical (ccw order)

deg = zeros(n,1); % face degree of each vertex
nb = zeros(0,n); % neighbors for each vertex
next = zeros(n,1); % "hash" table for ccw-directed edges

nt = size(tri,1);
p3 = [nt; 2*nt; 0]; % +1 mod 3  for linear indexing as ti+p3(j)
m3 = [2*nt; 0; nt]; % +2 mod 3

for i = 1:n % for each vertex i
    [ti, j] = find(tri==i); % triangles that use vertex i
    if ~isempty(ti)
        a = tri(ti+p3(j)); % edges are ab in ccw order around i
        b = tri(ti+m3(j));

        next(a) = 1; % each non-i edge points from vertex to next
        next(b) = 0; % check that every vertex in a is in b.
        if ~any(next(a)>0,1)    % ring is complete (closed)
            cnt = 0;
            k = a(1); % start anywhere and list vertices in ring
            next(a) = b; % each vertex in ring points to next
            while k>0 % collect list of nb vertices in ccw order
                cnt = cnt+1;
                nb(cnt, i) = k;
                tmpk = next(k);
                next(k) = 0; % clear next array as we go
                k = tmpk;
            end
            % At the end of this loop, the first vertex equals the last, so
            % we overwrite the first with the center of the ring.
            nb(1, i) = i;
            deg(i) = numel(ti)+1;

            if cnt ~= deg(i) % sanity check
                disp(['Warning: Unexpected ring count ' num2str([cnt deg(i)])])
            end
        end
    end
end

