function [tri, tdet] = orientTriangulation(tri, v)
% [tri, tdet] = orientTriangulation(tri, v)
% give canonical order for triangle vertices, which has two parts:
% 1. vertex with smallest x (breaking ties by y) is first,
% 2. other two are chosen so orientation determiant is positive:
% 1 x(tri(:,1)) y(tri(:,1))
% 1 x(tri(:,2)) y(tri(:,2))
% 1 x(tri(:,3)) y(tri(:,3))
% [t, d]=orientTriangulation([1 2 3; 2 3 4; 1 3 2; 4 2 1], [0 0; 1 0; 0 1; 1 1]) 
% should give [1 2 3; 3 2 4; 1 2 3; 1 2 4] , [1; 1; 1; 1]; % test calculation

[junk, irank] = sortrows(v); % determine permutation into increasing lex order
rank(irank,1) = 1:length(irank); % rank of vertex in lex order
[junk, perm] = sort(rank(tri),2); % permute triangles so lex min vertex is first
nt = size(tri,1);
for i = 1:nt
    tri(i,:) = tri(i,perm(i,:));
end

% this is the orientation determinant
tdet = (v(tri(:,2),1) - v(tri(:,1),1)).* (v(tri(:,3),2) - v(tri(:,1),2))...
    - (v(tri(:,3),1) - v(tri(:,1),1)).* (v(tri(:,2),2) - v(tri(:,1),2));
flip = tdet<0; % swap vertex 2,3 for any that are negative
if any(flip)
    tri(flip,:) = tri(flip, [1 3 2]);
    tdet(flip) = -tdet(flip);
end

if any(tdet==0) % Warn of degenerate (flat) triangles
    disp(['Warning: ' num2str(sum(tdet==0)) ' flat triangles'])
    disp(tri(tdet==0,:))
end

% Debug check that rank was correctly ordered
% if any(rank(tri(:,1))> min(rank(tri(:,2:3)), [], 2))
%     disp('oops in orientTriangulation');
% end
