function  [alpha,ind] = ptsInTri(v, Mtotal,u)
%-----------------------------------------------------------------------------
%             [alpha,ind]=ptsInTri(v,Mtotal,u)
%
%      V:  vector storing the coordinates of a triangle
%          with lex smallest first, oriented ccw
% Mtotal: saved orientation determinant (positive)
%      u:  n x 3-d vector, the coordinates of all points 
%
%  alpha: barycentric coordinates of points inside. A point is considered
%         inside if it on the bottom edges, except for the extreme vertices.
%  ind:  indices of points inside.  
%
%  Note: to make this function work, triangle must be canonical.
%         see orientTriangulation.m
%-----------------------------------------------------------------------------

down = v(2,1) < v(3,1) || (v(2,1)==v(3,1) && v(2,2) < v(3,2)); % downward triangle
miny = min(v(:,2));
maxy = max(v(:,2)); % filter outside of triangle bounding box
ind = find(u(:,1) > v(1,1) & u(:,1) <= v(2+down,1) & u(:,2)>=miny & u(:,2)<=maxy);

x = u(ind,1)-v(1,1); % compute barycentric coordinates
y = u(ind,2)-v(1,2);
alpha(:,3) = ((v(2,1)-v(1,1))*y-(v(2,2)-v(1,2))*x)/Mtotal; % det 1,2,u - index 3
alpha(:,2) = ((v(1,1)-v(3,1))*y-(v(1,2)-v(3,2))*x)/Mtotal; % det 3,1,u - index 2
alpha(:,1) = ((v(3,1)-v(2,1))*(u(ind,2)-v(2,2)) - (v(3,2)-v(2,2))*(u(ind,1)-v(2,1)))/Mtotal; % det 2,3,u - index 1

if down % bad points are those outside of or on top edges of triangle.
    bad = alpha(:,2) <= 0 | alpha(:,1) < 0  | alpha(:,3)<0;
else
    bad = alpha(:,2) <= 0 | alpha(:,1) <= 0 | alpha(:,3)<0;
end
ind(bad) = [];
alpha(bad,:) = [];
