function [Znew,Pnew] = refineOneRing(Z,P)
% For Z,P parameters of one-ring in nb order (1st is center, rest ccw)
% compute the new Z,P parameters for the refined ring
% Z,P contains a row for each point, 
% each row being a vector of z, p1...pk coefficients.
% (The first is the z coordinate for that point, but it is
% treated as a z coefficient whose multiplier will be 1.)

% Charles & Wenjie's subdivision matrices; used below
% Bmx = [3/8, 0; -47/512, 69/512]; % refine
% Cmx = [1/8, 0; -17/512, -5/512];
% Pmx = [1, -435/256; 0, -91/256]; % avg
% Dmx = [0, 145/512; 0, -45/512];
% an = 5/8 - ( 3/8 + 1/4 * cos( 2*pi / n ) )^2;   % avg, deg~=6
% Qnmx = [1, an*(-145/31); 0, -an];               %<== P
% Anmx = (512/31) * an * Dmx;                     %<== D

[n,m] = size(Z);
Znew = zeros(n,m);
Pnew = zeros(n,m);
ctr = 1; % center of one-ring
cn = ones(n-1,1);
nb = 2:n; % neighbors of 1
p1 = [3:n 2]; % next neighbor
m1 = [n 2:n-1]; % previous neighbor

% refine to put points on edge to neighbors B*(ctr+nb) + C*(next+prev)
Znew(nb,:) = 3/8*(Z(cn,:)+Z(nb,:)) -47/512*(P(cn,:)+P(nb,:))...
            +1/8*(Z(m1,:)+Z(p1,:)) -17/512*(P(m1,:)+P(p1,:));
Pnew(nb,:) =  +69/512*(P(cn,:)+P(nb,:))...
               -5/512*(P(m1,:)+P(p1,:));

% adjust existing points P values, depending on degree
Znew(ctr,:) = Z(ctr,:);
if n==7 % ordinary point
    Pnew(ctr,:) = -435/256*Z(ctr,:)     -91/256*P(ctr,:)...
                  +145/512*sum(Z(nb,:)) -45/512*sum(P(nb,:));
else 
    an = 5/8 - ( 3/8 + 1/4 * cos( 2*pi / n ) )^2;
    Pnew(ctr,:) = -145/31*an*Z(ctr,:)     -an*P(ctr,:)...
                  +145/31*an*sum(Z(nb,:)) -45/31*an*sum(P(nb,:));
end

%% Test code
% P=[zeros(7,1)  eye(7)];
% Z = [[2; ones(6,1)] zeros(7)];
% [Znew,Pnew]= refineOneRing(Z,P);
% format short g
% Znew(1:2,1:4)
% Pnew(1:2,1:4)
%%            2            0            0            0 %Znew
%        1.375    -0.091797    -0.091797    -0.033203
%      -1.6992     -0.35547    -0.087891    -0.087891 %Pnew
%            0      0.13477      0.13477   -0.0097656
